@@ -1,13 +1,16 @@
+from abc import ABCMeta, abstractmethod
 from typing import Tuple, Any
 
 from .ITemperatures import ITemperatures
 
 
-class ICooling(ITemperatures):
+class ICooling(ITemperatures, metaclass=ABCMeta):
     """The module can control the cooling of a device."""
-    __module__ = 'pyobs.interfaces'
 
-    def get_cooling_status(self, **kwargs: Any) -> Tuple[bool, float, float]:
+    __module__ = "pyobs.interfaces"
+
+    @abstractmethod
+    async def get_cooling_status(self, **kwargs: Any) -> Tuple[bool, float, float]:
         """Returns the current status for the cooling.
 
         Returns:
@@ -16,9 +19,10 @@ class ICooling(ITemperatures):
                 SetPoint: Setpoint for the cooling in celsius.
                 Power:    Current cooling power in percent or None.
         """
-        raise NotImplementedError
+        ...
 
-    def set_cooling(self, enabled: bool, setpoint: float, **kwargs: Any) -> None:
+    @abstractmethod
+    async def set_cooling(self, enabled: bool, setpoint: float, **kwargs: Any) -> None:
         """Enables/disables cooling and sets setpoint.
 
         Args:
@@ -28,7 +32,7 @@ class ICooling(ITemperatures):
         Raises:
             ValueError: If cooling could not be set.
         """
-        raise NotImplementedError
+        ...
 
 
-__all__ = ['ICooling']
+__all__ = ["ICooling"]
