@@ -1,14 +1,17 @@
+from abc import ABCMeta, abstractmethod
 from typing import Tuple, Any
 
 from .interface import Interface
 
 
-class IOffsetsAltAz(Interface):
+class IOffsetsAltAz(Interface, metaclass=ABCMeta):
     """The module supports Alt/Az offsets, usually combined with :class:`~pyobs.interfaces.ITelescope` and
     :class:`~pyobs.interfaces.IAltAz`."""
-    __module__ = 'pyobs.interfaces'
 
-    def set_offsets_altaz(self, dalt: float, daz: float, **kwargs: Any) -> None:
+    __module__ = "pyobs.interfaces"
+
+    @abstractmethod
+    async def set_offsets_altaz(self, dalt: float, daz: float, **kwargs: Any) -> None:
         """Move an Alt/Az offset.
 
         Args:
@@ -18,15 +21,16 @@ class IOffsetsAltAz(Interface):
         Raises:
             ValueError: If offset could not be set.
         """
-        raise NotImplementedError
+        ...
 
-    def get_offsets_altaz(self, **kwargs: Any) -> Tuple[float, float]:
+    @abstractmethod
+    async def get_offsets_altaz(self, **kwargs: Any) -> Tuple[float, float]:
         """Get Alt/Az offset.
 
         Returns:
             Tuple with alt and az offsets.
         """
-        raise NotImplementedError
+        ...
 
 
-__all__ = ['IOffsetsAltAz']
+__all__ = ["IOffsetsAltAz"]
